from datetime import datetime
import time
import ssl
import json
from typing import Type, TypeVar, List, Dict, Optional
import logging

from pyeconet.errors import (
    PyeconetError,
    InvalidCredentialsError,
    GenericHTTPError,
    InvalidResponseFormat,
)
from pyeconet.equipment import Equipment, EquipmentType
from pyeconet.equipment.water_heater import WaterHeater
from pyeconet.equipment.thermostat import Thermostat

from aiohttp import ClientSession, ClientTimeout
from aiohttp.client_exceptions import ClientError
import paho.mqtt.client as mqtt

HOST = "rheem.clearblade.com"
REST_URL = f"https://{HOST}/api/v/1"
CLEAR_BLADE_SYSTEM_KEY = "e2e699cb0bb0bbb88fc8858cb5a401"
CLEAR_BLADE_SYSTEM_SECRET = "E2E699CB0BE6C6FADDB1B0BC9A20"
HEADERS = {
    "ClearBlade-SystemKey": CLEAR_BLADE_SYSTEM_KEY,
    "ClearBlade-SystemSecret": CLEAR_BLADE_SYSTEM_SECRET,
    "Content-Type": "application/json; charset=UTF-8",
}

_LOGGER = logging.getLogger(__name__)

ApiType = TypeVar("ApiType", bound="EcoNetApiInterface")


def _create_ssl_context() -> ssl.SSLContext:
    """Create a SSL context for the MQTT connection."""
    context = ssl.SSLContext(ssl.PROTOCOL_TLS)
    context.load_default_certs()
    return context


_SSL_CONTEXT = _create_ssl_context()


class EcoNetApiInterface:
    """
    API interface object.
    """

    def __init__(
            self, email: str, password: str, account_id: str = None, user_token: str = None
    ) -> None:
        """
        Create the EcoNet API interface object.
        Args:
            email (str): EcoNet account email address.
            password (str): EcoNet account password.

        """
        self.email: str = email
        self.password: str = password
        self._user_token: str = user_token
        self._account_id: str = account_id
        self._locations: List = []
        self._equipment: Dict = {}
        self._mqtt_client = None

    @property
    def user_token(self) -> str:
        """Return the current user token"""
        return self._user_token

    @property
    def account_id(self) -> str:
        """Return the current user token"""
        return self._account_id

    @classmethod
    async def login(cls: Type[ApiType], email: str, password: str) -> ApiType:
        """Create an EcoNetApiInterface object using email and password
        Args:
            email (str): EcoNet account email address.
            password (str): EcoNet account password.

        """
        this_class = cls(email, password)
        await this_class._authenticate({"email": email, "password": password})
        return this_class

    def check_mode_enum(self, equip, enumtext=None):
        # Fix enumeration of Emergency Heat in Thermostat, maybe others?
        if "@MODE" in equip and isinstance(equip["@MODE"], Dict):
            if 'constraints' in equip["@MODE"]:
                enumtext = equip["@MODE"]['constraints']['enumText']
            status = equip["@MODE"].get('status')
            if enumtext and status:
                value = equip["@MODE"]['value']
                try:
                    enumtext_index_by_status = enumtext.index(status)
                    if value != enumtext_index_by_status:
                        _LOGGER.debug("Enum value mismatch: "
                                      f"{enumtext[value]} != "
                                      f"{equip['@MODE']['status']}")
                        equip["@MODE"]['value'] = enumtext_index_by_status
                except ValueError:
                    # friedrich seems to return cool vs cooling which causes this issue but the value (index)
                    # is still right so maybe we can ignore it?
                    _LOGGER.debug("Enum value mismatch: "
                                  f"{enumtext[value]} != "
                                  f"{equip['@MODE']['status']}")

        return equip, enumtext

    def check_update_enum(self, equip, update):
        # Update messages only have the status and value, so get the enumtext
        equip, enumtext = self.check_mode_enum(equip)
        # Fix the update
        update, __ = self.check_mode_enum(update, enumtext)
        return equip, update

    def subscribe(self):
        """Subscribe to the MQTT updates"""
        if not self._equipment:
            _LOGGER.error(
                "Equipment list is empty, did you call get_equipment before subscribing?"
            )
            return False

        self._mqtt_client = mqtt.Client(
            callback_api_version=mqtt.CallbackAPIVersion.VERSION1,
            client_id=self._get_client_id(),
            clean_session=True,
            userdata=None,
            protocol=mqtt.MQTTv311,
        )
        self._mqtt_client.username_pw_set(
            self._user_token, password=CLEAR_BLADE_SYSTEM_KEY
        )
        self._mqtt_client.enable_logger()

        self._mqtt_client.tls_set_context(_SSL_CONTEXT)
        self._mqtt_client.tls_insecure_set(False)

        self._mqtt_client.on_connect = self._on_connect
        self._mqtt_client.on_message = self._on_message
        self._mqtt_client.on_disconnect = self._on_disconnect
        self._mqtt_client.connect_async(HOST, 1884, 60)
        self._mqtt_client.loop_start()

    def publish(self, payload: Dict, device_id: str, serial_number: str):
        """Publish payload to the specified topic"""
        date_time = datetime.now().strftime("%Y-%m-%dT%H:%M:%S")
        transaction_id = f"ANDROID_{date_time}"
        publish_payload = {
            "transactionId": transaction_id,
            "device_name": device_id,
            "serial_number": serial_number,
        }
        publish_payload.update(payload)
        self._mqtt_client.publish(
            f"user/{self._account_id}/device/desired",
            payload=json.dumps(publish_payload),
        )

    def unsubscribe(self) -> None:
        self._mqtt_client.loop_stop()

    def _get_client_id(self) -> str:
        time_string = str(time.time()).replace(".", "")[:13]
        return f"{self.email}{time_string}_android"

    async def _get_equipment(self) -> None:
        """Get a list of all the equipment for this user"""
        _locations: List = await self._get_location()
        for _location in _locations:
            # They spelled it wrong...
            for _equip in _location.get("equiptments"):
                # Early exit if server returned error code
                if "error" in _equip:
                    _LOGGER.debug("EcoNet equipment error message"
                                  f": {_equip.get('error')}")
                    continue
                _equip, __ = self.check_mode_enum(_equip)
                _equip_obj: Equipment = None
                if (
                        Equipment._coerce_type_from_string(_equip.get("device_type"))
                        == EquipmentType.WATER_HEATER
                ):
                    _equip_obj = WaterHeater(_equip, self)
                    self._equipment[_equip_obj.serial_number] = _equip_obj
                elif (
                        Equipment._coerce_type_from_string(_equip.get("device_type"))
                        == EquipmentType.THERMOSTAT
                ):
                    _equip_obj = Thermostat(_equip, self)
                    self._equipment[_equip_obj.serial_number] = _equip_obj
                    for zoning_device in _equip.get("zoning_devices", []):
                        _equip_obj = Thermostat(zoning_device, self)
                        self._equipment[_equip_obj.serial_number] = _equip_obj

    async def refresh_equipment(self) -> None:
        """Get a list of all the equipment for this user"""
        _locations: List = await self._get_location()
        for _location in _locations:
            # They spelled it wrong...
            for _equip in _location.get("equiptments"):
                serial = _equip.get("serial_number")
                equipment = self._equipment.get(serial)
                if equipment:
                    _equip, __ = self.check_mode_enum(_equip)
                    equipment.update_equipment_info(_equip)

    async def get_equipment_by_type(self, equipment_type: List) -> Dict:
        """Get a list of equipment by the equipment EquipmentType"""
        if not self._equipment:
            await self._get_equipment()
        _equipment = {}
        for _equip_type in equipment_type:
            _equipment[_equip_type] = []
        for value in self._equipment.values():
            if value.type in equipment_type:
                _equipment[value.type].append(value)
        return _equipment

    async def _get_location(self) -> List[Dict]:
        _headers = HEADERS.copy()
        _headers["ClearBlade-UserToken"] = self._user_token
        payload = {"resource": "friedrich"}
        # payload = {"location_only": False, "type": "com.econet.econetconsumerandroid", "version": "6.0.0-375-01b4870e"}

        _session = ClientSession()
        try:
            async with _session.post(
                    f"{REST_URL}/code/{CLEAR_BLADE_SYSTEM_KEY}/getUserDataForApp", json=payload, headers=_headers
            ) as resp:
                if resp.status == 200:
                    _json = await resp.json()
                    _LOGGER.debug(json.dumps(_json, indent=2))
                    if _json.get("success"):
                        self._locations = _json["results"]["locations"]
                        return self._locations
                    else:
                        raise InvalidResponseFormat()
                else:
                    raise GenericHTTPError(resp.status)
        except ClientError as err:
            raise err
        finally:
            await _session.close()

    async def get_dynamic_action(self, payload: Dict) -> Dict:
        _headers = HEADERS.copy()
        _headers["ClearBlade-UserToken"] = self._user_token

        _session = ClientSession()
        try:
            async with _session.post(
                    f"{REST_URL}/code/{CLEAR_BLADE_SYSTEM_KEY}/dynamicAction",
                    json=payload,
                    headers=_headers,
            ) as resp:
                if resp.status == 200:
                    _json = await resp.json()
                    _LOGGER.debug(json.dumps(_json, indent=2))
                    if _json.get("success"):
                        return _json

                    raise InvalidResponseFormat()

                raise GenericHTTPError(resp.status)
        except ClientError as err:
            raise err
        finally:
            await _session.close()

    async def _authenticate(self, payload: dict) -> None:

        _session = ClientSession()
        try:
            async with _session.post(
                    f"{REST_URL}/user/auth", json=payload, headers=HEADERS
            ) as resp:
                if resp.status == 200:
                    _json = await resp.json()
                    _LOGGER.debug(json.dumps(_json, indent=2))
                    if _json.get("options")["success"]:
                        self._user_token = _json.get("user_token")
                        self._account_id = _json.get("options").get("account_id")
                    else:
                        raise InvalidCredentialsError(_json.get("options")["message"])
                else:
                    raise GenericHTTPError(resp.status)
        finally:
            await _session.close()

    def _on_connect(self, client, userdata, flags, rc):
        _LOGGER.debug(f"Connected with result code: {str(rc)}")
        client.subscribe(f"user/{self._account_id}/device/reported")
        client.subscribe(f"user/{self._account_id}/device/desired")

    def _on_disconnect(self, client, userdata, rc):
        _LOGGER.debug(f"Disconnected with result code: {str(rc)}")
        if rc != 0:
            _LOGGER.error("EcoNet MQTT unexpected disconnect. Attempting to reconnect.")
            client.reconnect()

    def _on_message(self, client, userdata, msg):
        """When a MQTT message comes in push that update to the specified equipment"""
        try:
            unpacked_json = json.loads(msg.payload)
            _LOGGER.debug("MQTT message from topic: %s", msg.topic)
            _LOGGER.debug(json.dumps(unpacked_json, indent=2))
            _name = unpacked_json.get("device_name")
            _serial = unpacked_json.get("serial_number")
            key = _serial
            _equipment = self._equipment.get(key)
            if _equipment is not None:
                _equipment._equipment_info, unpacked_json = self.check_update_enum(_equipment._equipment_info,
                                                                                   unpacked_json)
                _equipment.update_equipment_info(unpacked_json)
            # Nasty hack to push signal updates to the device it belongs to
            elif "@SIGNAL" in str(unpacked_json):
                for _equipment in self._equipment.values():
                    if _equipment.device_id == _name:
                        # Don't break after update for multi zone HVAC systems
                        _equipment.update_equipment_info(unpacked_json)
            else:
                _LOGGER.debug(
                    "Received update for non-existent equipment with device name: %s and serial number %s",
                    _name,
                    _serial,
                )
        except Exception as e:
            _LOGGER.exception(e)
            _LOGGER.error("Failed to parse the following MQTT message: %s", msg.payload)



"""EcoNet thermostat"""

import logging
from enum import Enum
from typing import Tuple, Union, List, Optional

from . import Equipment

_LOGGER = logging.getLogger(__name__)


class ThermostatOperationMode(Enum):
    """Define the operation mode"""

    OFF = 1
    HEATING = 2
    COOLING = 3
    AUTO = 4
    FAN_ONLY = 5
    EMERGENCY_HEAT = 6
    UNKNOWN = 99

    @staticmethod
    def by_string(str_value: str):
        """Convert a string to a supported OperationMode"""
        _cleaned_string = str_value.rstrip().replace(" ", "").upper()
        if _cleaned_string == ThermostatOperationMode.OFF.name.upper():
            return ThermostatOperationMode.OFF
        elif _cleaned_string == ThermostatOperationMode.HEATING.name.upper():
            return ThermostatOperationMode.HEATING
        elif _cleaned_string == ThermostatOperationMode.COOLING.name.upper():
            return ThermostatOperationMode.COOLING
        elif _cleaned_string == ThermostatOperationMode.AUTO.name.upper():
            return ThermostatOperationMode.AUTO
        elif (
            _cleaned_string
            == ThermostatOperationMode.FAN_ONLY.name.replace("_", "").upper()
        ):
            return ThermostatOperationMode.FAN_ONLY
        elif (
            _cleaned_string
            == ThermostatOperationMode.EMERGENCY_HEAT.name.replace("_", "").upper()
        ):
            return ThermostatOperationMode.EMERGENCY_HEAT
        else:
            _LOGGER.error("Unknown mode: [%s]", str_value)
            return ThermostatOperationMode.UNKNOWN


class ThermostatFanSpeed(Enum):
    """Define the fan speeds"""

    AUTO = 1
    LOW = 2
    MEDLO = 3
    MEDIUM = 4
    MEDHI = 5
    HIGH = 6
    MAX = 7
    UNKNOWN = 99

    @staticmethod
    def by_string(str_value: str):
        """Convert a string to a supported OperationMode"""
        _cleaned_string = str_value.rstrip().replace(" ", "_").replace(".", "").upper()
        if _cleaned_string == ThermostatFanSpeed.AUTO.name.upper():
            return ThermostatFanSpeed.AUTO
        elif _cleaned_string == ThermostatFanSpeed.LOW.name.upper():
            return ThermostatFanSpeed.LOW
        elif _cleaned_string == ThermostatFanSpeed.MEDLO.name.upper():
            return ThermostatFanSpeed.MEDLO
        elif _cleaned_string == ThermostatFanSpeed.MEDIUM.name.upper():
            return ThermostatFanSpeed.MEDIUM
        elif _cleaned_string == ThermostatFanSpeed.MEDHI.name.upper():
            return ThermostatFanSpeed.MEDHI
        elif _cleaned_string == ThermostatFanSpeed.HIGH.name.upper():
            return ThermostatFanSpeed.HIGH
        elif _cleaned_string == ThermostatFanSpeed.MAX.name.upper():
            return ThermostatFanSpeed.MAX
        else:
            _LOGGER.error("Unknown fan speed: [%s]", str_value)
            return ThermostatFanSpeed.UNKNOWN

class ThermostatFanMode(Enum):
    """Define the fan modes"""

    AUTO = 1
    ON_CONTINUOUS = 2
    UNKNOWN = 99

    @staticmethod
    def by_string(str_value: str):
        """Convert a string to a supported OperationMode"""
        _cleaned_string = str_value.rstrip().replace(" ", "_").replace(".", "").replace("/", "_").upper()
        if _cleaned_string == ThermostatFanMode.AUTO.name.upper():
            return ThermostatFanMode.AUTO
        elif _cleaned_string == ThermostatFanMode.ON_CONTINUOUS.name.upper():
            return ThermostatFanMode.ON_CONTINUOUS
        else:
            _LOGGER.error("Unknown fan mode: [%s]", str_value)
            return ThermostatFanMode.UNKNOWN

class Thermostat(Equipment):
    @property
    def running(self) -> bool | None:
        """Return if the thermostat is running or not"""
        if self.running_state is not None:
            return self._equipment_info.get("@RUNNINGSTATUS") != ""
        else:
            return None

    @property
    def running_state(self) -> str | None:
        """Return the raw running status value"""
        return self._equipment_info.get("@RUNNINGSTATUS")


    @property
    def beep_enabled(self) -> bool | None:
        """Return if thermostat beep is enabled or not"""
        beep = self._equipment_info.get("@BEEP")
        if beep is not None:
            return beep["value"] == 1
        else:
            return None

    @property
    def supports_humidifier(self) -> bool:
        de_hum_enabled = self._equipment_info.get("@DEHUMENABLE")
        if de_hum_enabled:
            return de_hum_enabled.get("constraints") is not None
        else:
            return False

    @property
    def cool_set_point(self) -> int:
        """Return the current cool set point"""
        return self._equipment_info.get("@COOLSETPOINT")["value"]

    @property
    def cool_set_point_limits(self) -> Tuple:
        """Returns a tuple of the lower limit and upper limit for the cool set point"""
        set_point = self._equipment_info.get("@COOLSETPOINT")["constraints"]
        return set_point["lowerLimit"], set_point["upperLimit"]

    @property
    def heat_set_point(self) -> int:
        return self._equipment_info.get("@HEATSETPOINT")["value"]

    @property
    def heat_set_point_limits(self) -> Tuple:
        """Returns a tuple of the lower limit and upper limit for the heat set point"""
        set_point = self._equipment_info.get("@HEATSETPOINT")["constraints"]
        return set_point["lowerLimit"], set_point["upperLimit"]

    @property
    def deadband(self) -> int:
        return self._equipment_info.get("@DEADBAND")["value"]

    @property
    def deadband_set_point_limits(self) -> Tuple:
        """Returns a tuple of the lower limit and upper limit for the cool set point"""
        set_point = self._equipment_info.get("@DEADBAND")["constraints"]
        return set_point["lowerLimit"], set_point["upperLimit"]

    @property
    def dehumidifier_set_point(self) -> int:
        return self._equipment_info.get("@DEHUMSETPOINT")["value"]

    @property
    def dehumidifier_set_point_limits(self) -> Tuple[int, int]:
        """Returns a tuple of the lower limit and upper limit for the dehumidifier set point"""
        set_point = self._equipment_info.get("@DEHUMSETPOINT")["constraints"]
        return set_point["lowerLimit"], set_point["upperLimit"]

    @property
    def dehumidifier_enabled(self) -> bool:
        return self._equipment_info.get("@DEHUMENABLE")["value"] == 1

    @property
    def supports_fan_mode(self) -> bool:
        return self._equipment_info.get("@FANMODE") is not None

    def set_dehumidifier_set_point(self, humidity):
        """Set the provided humidity"""
        lower, upper = self.dehumidifier_set_point_limits
        if lower <= humidity <= upper:
            payload = {"@DEHUMSETPOINT": humidity}
            self._api.publish(payload, self.device_id, self.serial_number)
        else:
            _LOGGER.error(
                "Set point out of range. Lower: %s Upper: %s Humidity set point: %s",
                lower,
                upper,
                humidity,
            )

    @property
    def zone_id(self) -> Union[str, None]:
        """Return the zode id"""
        return self._equipment_info.get("@ZONE_ID_NAME")

    @property
    def humidity(self) -> Union[int, None]:
        """Returns the current humidity"""
        humidity = self._equipment_info.get("@HUMIDITY")
        if humidity:
            return humidity["value"]
        else:
            return None

    @property
    def screen_locked(self) -> Union[bool, None]:
        screen_lock = self._equipment_info.get("@SCREENLOCK")
        if screen_lock:
            return screen_lock["value"] == 1
        else:
            return None

    @property
    def modes(self) -> List[ThermostatOperationMode]:
        """Return a list of supported operation modes"""
        _supported_modes = []
        _modes = self._equipment_info.get("@MODE")["constraints"]["enumText"]
        for _mode in _modes:
            _op_mode = ThermostatOperationMode.by_string(_mode)
            if _op_mode is not ThermostatOperationMode.UNKNOWN:
                _supported_modes.append(_op_mode)
        return _supported_modes

    @property
    def mode(self) -> Union[ThermostatOperationMode, None]:
        """Return the current mode"""
        return self.modes[self._equipment_info.get("@MODE")["value"]]

    @property
    def set_point_limits(self) -> Tuple:
        """
        Returns a tuple of the lower limit and upper limit for the set point.

        Thermostat set points are too extreme -100 - 200 setting reasonable limits.
        """
        cool_set_point = self._equipment_info.get("@COOLSETPOINT")["constraints"]
        heat_set_point = self._equipment_info.get("@HEATSETPOINT")["constraints"]
        if self.mode == ThermostatOperationMode.COOLING:
            return cool_set_point["lowerLimit"], cool_set_point["upperLimit"]
        elif self.mode == ThermostatOperationMode.AUTO or self.mode == ThermostatOperationMode.FAN_ONLY:
            return cool_set_point["lowerLimit"], heat_set_point["upperLimit"]
        else:
            return heat_set_point["lowerLimit"], heat_set_point["upperLimit"]

    def set_mode(self, mode: ThermostatOperationMode):
        """Set the provided mode"""
        payload = {}
        text_modes = self._equipment_info["@MODE"]["constraints"]["enumText"]
        count = 0
        for text_mode in text_modes:
            if mode == ThermostatOperationMode.by_string(text_mode):
                payload["@MODE"] = count
            count = count + 1
        self._api.publish(payload, self.device_id, self.serial_number)

    @property
    def fan_speeds(self) -> List[ThermostatFanSpeed]:
        """Return a list of supported fan speeds"""
        _supported_modes = []
        _modes = self._equipment_info.get("@FANSPEED")["constraints"]["enumText"]
        for _mode in _modes:
            _op_mode = ThermostatFanSpeed.by_string(_mode)
            if _op_mode is not ThermostatFanSpeed.UNKNOWN:
                _supported_modes.append(_op_mode)
        return _supported_modes

    @property
    def fan_speed(self) -> Union[ThermostatFanSpeed, None]:
        """Return the current fan speed"""
        return self.fan_speeds[self._equipment_info.get("@FANSPEED")["value"]]

    def set_fan_speed(self, mode: ThermostatFanSpeed):
        """Set the provided mode"""
        payload = {}
        text_modes = self._equipment_info["@FANSPEED"]["constraints"]["enumText"]
        count = 0
        for text_mode in text_modes:
            if mode == ThermostatFanSpeed.by_string(text_mode):
                payload["@FANSPEED"] = count
            count = count + 1
        self._api.publish(payload, self.device_id, self.serial_number)

    @property
    def fan_modes(self) -> List[ThermostatFanMode]:
        """Return a list of supported fan modes"""
        _supported_modes = []
        _modes = self._equipment_info.get("@FANMODE")["constraints"]["enumText"]
        for _mode in _modes:
            _op_mode = ThermostatFanMode.by_string(_mode)
            if _op_mode is not ThermostatFanMode.UNKNOWN:
                _supported_modes.append(_op_mode)
        return _supported_modes

    @property
    def fan_mode(self) -> Union[ThermostatFanMode, None]:
        """Return the fan current mode"""
        return self.fan_modes[self._equipment_info.get("@FANMODE")["value"]]

    def set_fan_mode(self, mode: ThermostatFanMode):
        """Set the provided mode"""
        payload = {}
        text_modes = self._equipment_info["@FANMODE"]["constraints"]["enumText"]
        count = 0
        for text_mode in text_modes:
            if mode == ThermostatFanMode.by_string(text_mode):
                payload["@FANMODE"] = count
            count = count + 1
        self._api.publish(payload, self.device_id, self.serial_number)

    def set_set_point(self, target_temp, target_temp_cool, target_temp_heat):
        """Set the provided set points based on mode.

        if just target temp is passed the temp of the current mode will be set to target_temp, this isn't valid for auto

        If target_temp_cool or target_temp_heat are passed target_temp will be ignored.
        """
        cool_payload = {}
        heat_payload = {}
        if (
            target_temp_cool
            or target_temp
            and self.mode == ThermostatOperationMode.COOLING
        ):
            _temp = target_temp if target_temp else target_temp_cool
            if self.cool_set_point_limits[0] <= _temp <= self.cool_set_point_limits[1]:
                cool_payload["@COOLSETPOINT"] = _temp
            else:
                _LOGGER.error(
                    "Cool set point out of range. Lower: %s Upper: %s Cool set point: %s",
                    self.cool_set_point_limits[0],
                    self.cool_set_point_limits[1],
                    _temp,
                )
        if (
            target_temp_heat
            or target_temp
            and self.mode
            in [ThermostatOperationMode.HEATING, ThermostatOperationMode.EMERGENCY_HEAT]
        ):
            _temp = target_temp if target_temp else target_temp_heat
            if self.heat_set_point_limits[0] <= _temp <= self.heat_set_point_limits[1]:
                heat_payload["@HEATSETPOINT"] = _temp
            else:
                _LOGGER.error(
                    "Heat set point out of range. Lower: %s Upper: %s Heat set point: %s",
                    self.heat_set_point_limits[0],
                    self.heat_set_point_limits[1],
                    _temp,
                )

        has_set_temp = False
        if cool_payload and self.mode in [
            ThermostatOperationMode.AUTO,
            ThermostatOperationMode.COOLING,
        ]:
            self._api.publish(cool_payload, self.device_id, self.serial_number)
            has_set_temp = True
        if heat_payload and self.mode in [
            ThermostatOperationMode.AUTO,
            ThermostatOperationMode.HEATING,
            ThermostatOperationMode.EMERGENCY_HEAT,
        ]:
            self._api.publish(heat_payload, self.device_id, self.serial_number)
            has_set_temp = True
        if target_temp and not has_set_temp:
            payload = {}
            if self.mode == ThermostatOperationMode.COOLING:
                payload = cool_payload
            elif self.mode in [
                ThermostatOperationMode.HEATING,
                ThermostatOperationMode.EMERGENCY_HEAT,
            ]:
                payload = heat_payload
            else:
                _LOGGER.error(
                    "Can't auto determine set point to set when mode is: %s", self.mode
                )
            self._api.publish(payload, self.device_id, self.serial_number)



from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

# Get the long description from the README file
LONG_DESCRIPTION = (here / "README.md").read_text(encoding="utf-8")

VERSION = "0.1.29b6"

# Setting up
setup(
    name="pyeconet",
    version=VERSION,
    author="William Scanlon",
    description="Interface to the unofficial EcoNet API",
    long_description=LONG_DESCRIPTION,
    long_description_content_type="text/markdown",
    package_dir={"": "src"},
    packages=find_packages(
        where="src", exclude=["dist", "*.test", "*.test.*", "test.*", "test"]
    ),
    install_requires=["aiohttp>=3.11.11, <4", "paho-mqtt>=2.1.0, <3"],
    keywords=["econet", "rheem", "api"],
    python_requires=">=3.8, <4",
    url="https://github.com/w1ll1am23/pyeconet",
    project_urls={
        "Bug Reports": "https://github.com/w1ll1am23/pyeconet/issues",
        "Source": "https://github.com/w1ll1am23/pyeconet",
    },
    classifiers=[
        "License :: OSI Approved :: MIT License",
    ],
)
